import {
    Http, IDictionary, Metadata,
} from '@zeedhi/core';
import {
    DialogService, AlertService, FileInput, LoadingService, Form,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Utilities {
    public static alertMessage(message: string, type: string) {
        AlertService.show({
            name: 'basic-alert',
            text: message,
            top: true,
            left: false,
            timeout: 3000,
            color: type,
        });
    }

    public static showErrorMessage(error: any): void {
        console.error(error);
        let msg = '';
        if (error.response && error.response.data && error.response.data.error) {
            msg = error.response.data.error;
        } else {
            msg = error;
        }
        if (!msg) {
            msg = 'Ocorreu um erro inesperado. Por favor, entre em contato com o suporte.';
        }

        if (msg.includes('ORA-02292')) {
            msg = 'Exclusão não permitida! Um ou mais registros estão sendo usados.';
        }

        DialogService.show({
            name: 'error_dialog',
            type: 'error',
            title: 'Erro',
            text: msg,
        });
    }

    public static showSuccessMessage() {
        AlertService.show({
            name: 'basic-alert',
            text: 'Processo realizado com sucesso!',
            top: true,
            left: false,
            color: 'success',
            timeout: 2000,
            buttonsSlot: [
                {
                    name: 'undo-button',
                    component: 'ZdButton',
                    icon: true,
                    iconName: 'close',
                    color: 'white',
                    events: {
                        click: () => { AlertService.hide(); },
                    },
                },
            ],
        });
    }

    public static showDialog(titulo: string, tipo: any, msg: string) {
        DialogService.show({
            name: titulo,
            type: tipo,
            title: titulo,
            text: msg,
        });
    }

    public static messages(returnMessage: string, mensagem: string) {
        if (returnMessage === 'OK') {
            AlertService.show({
                name: 'basic-alert',
                text: mensagem,
                top: true,
                left: false,
                timeout: 3000,
                color: 'success',
            });
        } else {
            DialogService.show({
                name: 'warning-dialog',
                type: 'warning',
                title: 'Erro!',
                text: mensagem,
            });
        }
    }

    public static async buildArrayToUpload(nomeComponente: string) {
        const file = Metadata.getInstance<FileInput>(nomeComponente).value;
        console.warn(file);
        if (file) {
            const base64file = await this.toBase64(file);
            return [{
                b64File: base64file,
                lastModified: file.lastModified,
                name: file.name,
                size: file.size,
                type: file.type,
                webkitRelativePath: file.webkitRelativePath,
            }];
        }
        return [];
    }

    public static async toBase64(file: any) {
        return new Promise((resolve, reject) => {
            const reader = new FileReader();
            reader.readAsDataURL(file);
            reader.onload = () => resolve(reader.result);
            reader.onerror = (error) => reject(error);
        });
    }

    /* eslint-disable */
    public static showConfirmDeleteMessage(event: any, functionConfirm: string, title: string, msg: string = 'Deseja realmente excluir todos os registros selecionados?') {
        event.preventDefault();
        functionConfirm = '{{' + functionConfirm + '}}';
        DialogService.show({
            name: 'buttons-dialog',
            type: 'warning',
            title: title,
            text: msg,
            buttons: [
                {
                    name: 'button-no',
                    component: 'ZdButton',
                    label: 'Não',
                    events: {
                        click: () => DialogService.hide(),
                    },
                    outline: true,
                },
                {
                    name: 'button-yes',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: functionConfirm,
                    },
                },
            ],
        });
    }

    public static showDialogFunction(event: any, functionConfirm: string, title: string, msg: string) {
        event.preventDefault();
        functionConfirm = '{{' + functionConfirm + '}}';
        DialogService.show({
            name: 'Dialog-function',
            type: 'warning',
            title: title,
            text: msg,
            buttons: [
                {
                    name: 'button-no',
                    component: 'ZdButton',
                    label: 'Não',
                    events: {
                        click: () => DialogService.hide(),
                    },
                    outline: true,
                },
                {
                    name: 'button-yes',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: functionConfirm,
                    },
                },
            ],
        });
    }
    /* eslint-enable */

    public static save(route: string, params: {}) {
        LoadingService.show();
        return new Promise((resolve, reject) => {
            Http.post(route, params).then((response) => {
                this.showSuccessMessage();
                resolve(response);
            }).catch((error: any) => {
                this.showErrorMessage(error);
                reject(error);
            }).finally(() => {
                LoadingService.hide();
            });
        });
    }

    public static deleteInfoGrid(route: string, { component }: any) {
        LoadingService.show();
        return new Promise((resolve, reject) => {
            Http.post(route, {
                NRS: component.selectedRows,
            }).then((response: IDictionary) => {
                this.showSuccessMessage();
                component.datasource.get();
                resolve(response);
            }).catch((error: any) => {
                this.showErrorMessage(error);
                reject(error);
            }).finally(() => {
                component.selectedRows = [];
                LoadingService.hide();
            });
        });
    }

    public static validateForm(form: Form) {
        if (form.validate()) {
            return true;
        }
        this.alertMessage('Existem campos obrigatórios não preenchidos!', 'error');
        return false;
    }

    // MÓDULO ACL
    public static async timeout(time: number) {
        return new Promise((resolve) => setTimeout(resolve, time));
    }

    public static validateGrid(grid: TekGrid) {
        if (grid.isGridValid(true)) {
            return true;
        }
        this.alertMessage('Existem campos obrigatórios não preenchidos!', 'error');
        return false;
    }

    /* eslint-disable */
    public static showConfirmDisabledMessage(event: any, functionConfirm: string) {
        event.preventDefault();
        functionConfirm = `{{${functionConfirm}}}`;
        DialogService.show({
            name: 'buttons-dialog',
            type: 'warning',
            title: 'Inativar Registro',
            text: 'Deseja realmente inativar todos os registros selecionados?',
            buttons: [
                {
                    name: 'button-no',
                    component: 'ZdButton',
                    label: 'Não',
                    events: {
                        click: () => DialogService.hide(),
                    },
                    outline: true,
                },
                {
                    name: 'button-yes',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: functionConfirm,
                    },
                },
            ],
        });
    }
    /* eslint-enable */

    public static creatDate(dataInput: any) {
        const [data, horas] = dataInput.split(' ');
        const [dia, mes, ano] = data.split('/');
        const [hora, minuto] = horas.split(':');
        return new Date(
            ano,
            mes - 1,
            dia,
            hora,
            minuto,
        );
    }

    // public static validarCPF(cpf: string): boolean {
    //     const apenasNumeros = cpf.replace(/[^\d]+/g, ''); // Remove caracteres não numéricos

    //     if (apenasNumeros.length !== 11 || /^(\d)\1+$/.test(apenasNumeros)) return false;

    //     let soma = 0;
    //     let resto;

    //     for (let i = 0; i < 9; i++) soma += parseInt(apenasNumeros.charAt(i), 10) * (10 - i);
    //     resto = (soma * 10) % 11;
    //     if (resto === 10 || resto === 11) resto = 0;
    //     if (resto !== parseInt(apenasNumeros.charAt(9), 10)) return false;

    //     soma = 0;
    //     for (let i = 0; i < 10; i++) soma += parseInt(apenasNumeros.charAt(i), 10) * (11 - i);
    //     resto = (soma * 10) % 11;
    //     if (resto === 10 || resto === 11) resto = 0;
    //     if (resto !== parseInt(apenasNumeros.charAt(10), 10)) return false;

    //     return true;
    // }

    // public static validarCNPJ(cnpj: string): boolean {
    //     const apenasNumeros = cnpj.replace(/[^\d]+/g, '');

    //     if (apenasNumeros.length !== 14 || /^(\d)\1+$/.test(apenasNumeros)) return false;

    //     let tamanho = apenasNumeros.length - 2;
    //     const numeros = apenasNumeros.substring(0, tamanho);
    //     const digitos = apenasNumeros.substring(tamanho);
    //     let soma = 0;
    //     let pos = tamanho - 7;

    //     for (let i = tamanho; i >= 1; i--) {
    //         soma += parseInt(numeros.charAt(tamanho - i), 10) * pos--;
    //         if (pos < 2) pos = 9;
    //     }

    //     let resto = soma % 11;
    //     const digito1 = resto < 2 ? 0 : 11 - resto;
    //     if (digito1 !== parseInt(digitos.charAt(0), 10)) return false;

    //     tamanho += 1;
    //     const novosNumeros = apenasNumeros.substring(0, tamanho);
    //     soma = 0;
    //     pos = tamanho - 7;

    //     for (let i = tamanho; i >= 1; i--) {
    //         soma += parseInt(novosNumeros.charAt(tamanho - i), 10) * pos--;
    //         if (pos < 2) pos = 9;
    //     }

    //     resto = soma % 11;
    //     const digito2 = resto < 2 ? 0 : 11 - resto;
    //     if (digito2 !== parseInt(digitos.charAt(1), 10)) return false;

    //     return true;
    // }
}
